#ifndef TDIConstantForceH
#define TDIConstantForceH

// ==========================================================================
// File:         TDIConstantForce.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons, website),
//               Brian Austin (some source code, documentation)
// Description:  This file defines the TDIConstantForce Component 
//
// "TDx_Input_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Input_Library_Defns.H"
#include "TDx_Input_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDIConstantForce
// Description: The TDIConstantForce component wraps the DirectX 
//              DICONSTANTFORCE structure, which contains type specific 
//              information for force feedback effects that have the 
//              DIEFT_CONSTANTFORCE option set. 
//              
//              It is used to define a 'Constant Force', which is a force 
//              that has a defined magnitude and duration. 
//              
//              You can apply an envelope to a constant force but not an 
//              offset. 
//              
//              Only the magnitude of the force is defined by this component, 
//              the duration is set in TDIEffect::Duration. 
//              
//              Use this component by setting TDIEffect::TypeSpecificParams 
//              to refer to a single TDIConstantForce component. 
//              
//              To create a constant force, use the 
//              TDx_InputDevice::CreateEffect() method and pass the following 
//              as the GUID parameter: 
//              
//                  GUID_ConstantForce 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDIConstantForce : public TDIForce
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDIConstantForce : public TDIForce
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Magnitude
    // Description: The Magnitude property defines the magnitude of the force 
    //              to be applied. 
    //              
    //              This value can range from -10,000 to 10,000 where the 
    //              extreme values represent the maximum force output the 
    //              device is capable of in either direction along the axis. 
    //              
    //              If an envelope is being applied, this value defines the 
    //              force magnitude applied during the sustain period. 
    // ----------------------------------------------------------------------
    __property long Magnitude = { read=FGetMagnitude, write=FSetMagnitude, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDIConstantForce component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDIConstantForce method or 
    //              fget/fset. eg. DI_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DI_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of the 
    //              internal DICONSTANTFORCE structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDIConstantForce's internal DICONSTANTFORCE 
    //              structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDIConstantForce(TComponent* Owner); 
    virtual __fastcall ~TDIConstantForce(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DICONSTANTFORCE* Internal_DICONSTANTFORCE_Ptr = { 
    read=FGetInternal_DICONSTANTFORCE_Ptr, nodefault }; 

    void __fastcall Internal_DICONSTANTFORCE_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    long __fastcall FGetMagnitude(); 
    void __fastcall FSetMagnitude( long pMagnitude ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DICONSTANTFORCE* __fastcall FGetInternal_DICONSTANTFORCE_Ptr(); 
    void __fastcall FSetInternal_DICONSTANTFORCE( DICONSTANTFORCE* 
    pDICONSTANTFORCE ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    dword fSize; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DICONSTANTFORCE fDICONSTANTFORCE; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
