#ifndef TDICustomForceH
#define TDICustomForceH

// ==========================================================================
// File:         TDICustomForce.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons, website),
//               Brian Austin (some source code, documentation)
// Description:  This file defines the TDICustomForce Component 
//
// "TDx_Input_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Input_Library_Defns.H"
#include "TDx_Input_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDICustomForce
// Description: The TDICustomForce component wraps the DirectX DICUSTOMFORCE 
//              structure, which contains type specific information for a 
//              user defined force feedback effect that has the 
//              DIEFT_CUSTOMFORCE option set. 
//              
//              It is used to define 'Custom Force' effects, which are 
//              defined as an array of constant force values that are to be 
//              played back by the device. 
//              
//              Use this component by setting TDIEffect::TypeSpecificParams 
//              to refer to a single TDICustomForce component. 
//              
//              To create a custom force, use the 
//              TDx_InputDevice::CreateEffect() method and pass the following 
//              as the GUID parameter: 
//              
//                  GUID_CustomForce 
//              
//              TDICustomForce::SamplePeriod and TDIEffect::SamplePeriod must 
//              have the same value, which defines how long each force in the 
//              TDICustomForce::ForceData array will be played. 
//              
//              The whole the force will be played repeatedly for the period 
//              of time set in TDIEffect::Duration. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDICustomForce : public TDIForce
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDICustomForce : public TDIForce
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Channels
    // Description: The Channels property defines the number of channels 
    //              (axes) that are to be affected by the force. 
    //              
    //              The first channel is applied to the first axis associated 
    //              with the effect, the second to the second, etc. 
    //              Nothing will be associated with axes in surplus of the 
    //              number of channels. 
    //              
    //              When there's one channel the effect will be rotated in 
    //              the direction specified by TDIEffect::Direction. 
    //              Rotation of custom effects is not supported by all 
    //              devices. 
    //              When there are multiple channels rotation cannot take 
    //              place. 
    // ----------------------------------------------------------------------
    __property dword Channels = { read=FGetChannels, write=FSetChannels, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    SamplePeriod
    // Description: The SamplePeriod property defines the length of time, in 
    //              microseconds, each element in the 
    //              TDICustomForce::ForceData array should be played. 
    // ----------------------------------------------------------------------
    __property dword SamplePeriod = { read=FGetSamplePeriod, 
    write=FSetSamplePeriod, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Samples
    // Description: The Samples property defines the total number of elements 
    //              in the TDICustomForce::ForceData array. 
    //              This value must be a multiple of 
    //              TDICustomForce::Channels. 
    // ----------------------------------------------------------------------
    __property dword Samples = { read=FGetSamples, write=FSetSamples, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDICustomForce component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDICustomForce method or 
    //              fget/fset. eg. DI_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DI_OK }; 

    // ----------------------------------------------------------------------
    // Property:    ForceData
    // Description: The ForceData property references an array of force 
    //              magnitude values representing the custom force. 
    //              
    //              When multiple channels are being used the values will be 
    //              interleaved. 
    //              Thus the first element of the array will belong to the 
    //              first channel, the second to the second, etc. 
    // ----------------------------------------------------------------------
    __property long* ForceData = { read=FGetForceData, write=FSetForceData, 
    default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of the 
    //              internal DICUSTOMFORCE structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDICustomForce's internal DICUSTOMFORCE structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDICustomForce(TComponent* Owner); 
    virtual __fastcall ~TDICustomForce(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DICUSTOMFORCE* Internal_DICUSTOMFORCE_Ptr = { 
    read=FGetInternal_DICUSTOMFORCE_Ptr, nodefault }; 

    void __fastcall Internal_DICUSTOMFORCE_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetChannels(); 
    void __fastcall FSetChannels( dword pChannels ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    long* __fastcall FGetForceData(); 
    void __fastcall FSetForceData( long* pForceData ); 
    dword __fastcall FGetSamplePeriod(); 
    void __fastcall FSetSamplePeriod( dword pSamplePeriod ); 
    dword __fastcall FGetSamples(); 
    void __fastcall FSetSamples( dword pSamples ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DICUSTOMFORCE* __fastcall FGetInternal_DICUSTOMFORCE_Ptr(); 
    void __fastcall FSetInternal_DICUSTOMFORCE( DICUSTOMFORCE* pDICUSTOMFORCE 
    ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    dword fSize; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DICUSTOMFORCE fDICUSTOMFORCE; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
