#ifndef TDIEffectH
#define TDIEffectH

// ==========================================================================
// File:         TDIEffect.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons, website),
//               Brian Austin (some source code, documentation)
// Description:  This file defines the TDIEffect Component 
//
// "TDx_Input_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Input_Library_Defns.H"
#include "TDx_Input_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by TDIEffect properties.
#include "TDIEnvelope.H"

// ==========================================================================
// Set:         TDIEffectFlags_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TDIEffect::Flags property using the BCB Object Inspector. For 
//              detailed descriptions of each flag, see the Flags property 
//              description below. 
// ==========================================================================
enum TDIEffect_Flags_Flags
    {
    dieff_cartesian,
    dieff_objectids,
    dieff_objectoffsets,
    dieff_polar,
    dieff_spherical
    };
typedef Set< TDIEffect_Flags_Flags, 
             dieff_cartesian, 
             dieff_spherical >  TDIEffect_Flags_Set;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDIEffect
// Description: The TDIEffect component wraps the DirectX DIEFFECT structure 
//              used to set up effect parameters prior to downloading them to 
//              a device. 
//              
//              This component is used by the TDx_InputDevice::CreateEffect() 
//              method to initialize a new effect and by the 
//              TDx_InputEffect::SetParameters() and 
//              TDx_InputEffect::GetParameters() methods to check or change 
//              an effects parameters. 
//              
//              All the times, force values and modifiers such as envelopes 
//              that define an effect that is to be downloaded to a force 
//              feedback device are described or referenced by this 
//              component. 
//              
//              The TDIEnvelope and TDICondition components may be referenced 
//              by this component as an optional part of the effect 
//              definition. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDIEffect : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDIEffect : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Flags_
    // Description: The Flags property defines flags indicating which 
    //              coordinate system and axis interpretation should be used. 
    //              The described effect applies when the flag is set. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'Flags' property via the BCB Object Inspector. 
    //               For detailed descriptions of each flag, see the Flags 
    //              property description below. 
    // ----------------------------------------------------------------------
    __property TDIEffect_Flags_Set Flags_ = { read=FGetFlags_, 
    write=FSetFlags_, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    AxesCount
    // Description: The AxesCount property defines the number of axes 
    //              involved in this effect. 
    //              
    //              A value must be set for this property when changing the 
    //              axis or direction lists and the number of axes for an 
    //              effect cannot be changed once set. 
    // ----------------------------------------------------------------------
    __property dword AxesCount = { read=FGetAxesCount, write=FSetAxesCount, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Duration
    // Description: The Duration property defines the total duration, in 
    //              microseconds, the effect should be active. 
    //              
    //              This property can be set to INFINITE in which case the 
    //              effect has infinite duration. If there is an envelope 
    //              then the attack will be applied, followed by an infinite 
    //              sustain. 
    //              
    //              The INFINITE setting is most frequently used for 
    //              conditions since they respond to device motion rather 
    //              explicit effect calls. 
    // ----------------------------------------------------------------------
    __property dword Duration = { read=FGetDuration, write=FSetDuration, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Envelope
    // Description: The Envelope property references a TDIEnvelope component 
    //              defining the envelope to be used by the effect.   
    //              
    //              Not all effect types can use envelopes and if there is no 
    //              envelope, this property should be NULL. 
    // ----------------------------------------------------------------------
    __property TDIEnvelope* Envelope = { read=FGetEnvelope, 
    write=FSetEnvelope, default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    Gain
    // Description: The Gain property defines the gain to be applied to the 
    //              effect. 
    //              
    //              This value represents no force output at 0 and normal 
    //              force output at 10,000. 
    // ----------------------------------------------------------------------
    __property dword Gain = { read=FGetGain, write=FSetGain, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    SamplePeriod
    // Description: The SamplePeriod property defines the sample period, in 
    //              microseconds, the device should use to play back the 
    //              effect. 
    //              
    //              A setting of 0 results in the default sample period being 
    //              used and an unsupported rate will be rounded to the 
    //              closest setting the device can support. 
    // ----------------------------------------------------------------------
    __property dword SamplePeriod = { read=FGetSamplePeriod, 
    write=FSetSamplePeriod, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property defines the size, in bytes, of the 
    //              internal structure used by this component. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, 
    default=sizeof(DIEFFECT) }; 

    // ----------------------------------------------------------------------
    // Property:    StartDelay
    // Description: The StartDelay property defines how long, in 
    //              microseconds, to wait after the TDx_InputEffect::Start() 
    //              call is made before playing the effect. 
    // ----------------------------------------------------------------------
    __property dword StartDelay = { read=FGetStartDelay, 
    write=FSetStartDelay, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    TriggerButton
    // Description: The TriggerButton property indicates which button should 
    //              trigger the effect's playback. 
    //              
    //              A setting of DIEB_NOTRIGGER indicates that the effect is 
    //              not triggered by a button and the setting of 
    //              DIEFF_OBJECTIDS or DIEFF_OBJECTOFFSETS in 
    //              TDIEffect::Flags determines how this value is 
    //              interpreted. 
    // ----------------------------------------------------------------------
    __property dword TriggerButton = { read=FGetTriggerButton, 
    write=FSetTriggerButton, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    TriggerRepeatInterval
    // Description: The TriggerRepeatInterval property defines the how long, 
    //              in microseconds, to wait between playbacks of the effect 
    //              when the button is held down. 
    //              
    //              This property is only used if the effect is triggered by 
    //              a button press and repetition support is available. 
    //              Repetition support will be indicated by the 
    //              DIEP_TRIGGERREPEATINTERVAL flag being set in the 
    //              TDIEffectInfo::StaticParams associated with this effect. 
    //              
    //              A setting of INFINITE indicates there will be no effect 
    //              repetition while the button is held down. 
    // ----------------------------------------------------------------------
    __property dword TriggerRepeatInterval = { 
    read=FGetTriggerRepeatInterval, write=FSetTriggerRepeatInterval, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    TypeSpecificParamsSize
    // Description: The TypeSpecificParamsSize property defines total size, 
    //              in bytes, of the parameters referenced by 
    //              TDIEffect::TypeSpecificParams. Refer to the 
    //              TDIEffect::TypeSpecificParams property for examples of 
    //              how to calculate this value. 
    // ----------------------------------------------------------------------
    __property dword TypeSpecificParamsSize = { 
    read=FGetTypeSpecificParamsSize, write=FSetTypeSpecificParamsSize, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDIEffect component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Axes
    // Description: The Axes property references an array of 
    //              TDIEffect::AxesCount DWORD's identifying which axes the 
    //              effect is to be applied to. 
    //              
    //              The setting of DIEFF_OBJECTIDS or DIEFF_OBJECTOFFSETS in 
    //              TDIEffect::Flags will determine how the values contained 
    //              in the array are interpreted. 
    //              
    //              Up to 32 axes may be associated with a single effect, but 
    //              the list cannot be changed once it has been set. 
    // ----------------------------------------------------------------------
    __property dword* Axes = { read=FGetAxes, write=FSetAxes, default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    Direction
    // Description: The Direction property references an array of 
    //              TDIEffect::AxesCount LONG's containing either cartesian, 
    //              polar or spherical coordinates. 
    //              
    //              The setting of DIEFF_CARTESIAN, DIEFF_POLAR or 
    //              DIEFF_SPHERICAL in TDIEffect::Flags will determine how 
    //              the values contained in the array are interpreted. 
    //              
    //              Each value in the Direction array will be associated with 
    //              the corresponding value in the TDIEffect::Axes array for 
    //              cartesian coordinates. 
    //              
    //              Polar and spherical coordinates require one less element 
    //              than cartesian coordinates to indicate direction but the 
    //              Direction array must still contain TDIEffect::AxesCount 
    //              elements, with the last element set to zero. 
    // ----------------------------------------------------------------------
    __property long* Direction = { read=FGetDirection, write=FSetDirection, 
    default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDIEffect method or fget/fset. 
    //              eg. DI_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DI_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Flags
    // Description: The Flags property defines flags indicating which 
    //              coordinate system and axis interpretation should be used. 
    //              The described effect applies when the flag is set. 
    // Flags:       DIEFF_CARTESIAN -
    //                The contents of the array referenced by 
    //                TDIEffect::Direction should be interpreted as cartesian 
    //                coordinates. 
    //              DIEFF_OBJECTIDS -
    //                The contents of the arrays referenced by 
    //                TDIEffect::Axes and TDIEffect::TriggerButton should be 
    //                interpreted as object identifiers like those returned 
    //                by TDx_InputDevice::EnumObjects(). 
    //              DIEFF_OBJECTOFFSETS -
    //                The contents of the arrays referenced by 
    //                TDIEffect::Axes and TDIEffect::TriggerButton should be 
    //                interpreted as offsets into the data format selected 
    //                using TDx_InputDevice::SetDataFormat(). 
    //              DIEFF_POLAR -
    //                The contents of the array referenced by 
    //                TDIEffect::Direction should be interpreted as polar 
    //                coordinates. 
    //              DIEFF_SPHERICAL -
    //                The contents of the array referenced by 
    //                TDIEffect::Direction should be interpreted as spherical 
    //                coordinates. 
    // ----------------------------------------------------------------------
    __property dword Flags = { read=FGetFlags, write=FSetFlags, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    TypeSpecificParams
    // Description: The TypeSpecificParams property references the internal 
    //              structure of the appropriate TDICustomForce, TDIPeriodic, 
    //              TDIConstantForce, TDIRampForce or TDICondition component, 
    //              which in turn provide additional parameters appropriate 
    //              to the effect type. 
    //              
    //              NULL may be set when the effect has no type specific 
    //              parameters. 
    //              
    //              The TDIEffectInfo::EffectType property defines the effect 
    //              type :- DIEFT_CONDITION, DIEFT_CONSTANTFORCE, 
    //              DIEFT_CUSTOMFORCE, DIEFT_PERIODIC or DIEFT_RAMPFORCE. 
    //              
    //              For DIEFT_CONDITION effects, this property should 
    //              reference the internal array of DICONDITION structures 
    //              referenced by the TDICondition::Internal_DICONDITION_Ptr 
    //              property. 
    //              
    //              For example: 
    //              
    //                  void __fastcall TForm1::Button1Click(TObject* Sender) 
    //                  { 
    //                  // set multiwrapper to contain 5 x DICONDITION 
    //              structures. 
    //                  DICondition1->ArraySize = 5; 
    //                  . 
    //                  . 
    //                  . 
    //                  // set effect type 
    //                  DIEffectInfo1->EffectType = DIEFT_CONDITION; 
    //              
    //                  // set the array size 
    //                  DIEffect1->TypeSpecificParamsSize = 5 * 
    //              DICondition1->Size; 
    //                  // or you could set it like this 
    //                  DIEffect1->TypeSpecificParamsSize = 
    //              DICondition1->SizeAll; 
    //                  // or even like this 
    //                  DIEffect1->TypeSpecificParamsSize = 
    //              5*sizeof(DICONDITION); 
    //              
    //                  // point to first of the internal DICONDITION 
    //              structures in the DICondition1 component 
    //                  DIEffect1->TypeSpecificParams = 
    //              DICondition1->Internal_DICONDITION_Ptr[0]; 
    //                  } 
    //              
    //              When only one condition is being used the direction will 
    //              be obtained from TDIEffect::Direction. Otherwise, there 
    //              must be one internal DICONDITION structure for each axis 
    //              affected, in the same order as TDIEffect::Axes, the 
    //              effect should not be rotated and the following values 
    //              used in the TDIEffect::Direction array: 
    //              
    //                  Spherical coordinates : 0, 0, etc 
    //                  Polar coordinates : 9000, 0, etc 
    //                  Cartesian coordinates 1, 0, etc 
    //              
    //              When using the other effect types, this property should 
    //              reference the internal structure of the appropriate 
    //              wrapper component. 
    //              
    //              For example: 
    //              
    //                  void __fastcall TForm1::Button1Click(TObject* Sender) 
    //                  { 
    //                  // set effect type 
    //                  DIEffectInfo1->EffectType = DIEFT_CONSTANTFORCE; 
    //                 // set the size 
    //                  DIEffect1->TypeSpecificParamsSize = 
    //              DIConstantForce1->Size; 
    //                  // point to the internal DICONSTANTFORCE structure of 
    //              DIConstantForce1 
    //                  DIEffect1->TypeSpecificParams = 
    //              DIConstantForce1->Internal_DICONSTANTFORCE_Ptr; 
    //                  } 
    // ----------------------------------------------------------------------
    __property void* TypeSpecificParams = { read=FGetTypeSpecificParams, 
    write=FSetTypeSpecificParams, default=NULL }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDIEffect's internal DIEFFECT structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Property:    Flags_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the Flags property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* Flags_Strings = { read=FGetFlags_Strings };

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDIEffect(TComponent* Owner); 
    virtual __fastcall ~TDIEffect(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DIEFFECT* Internal_DIEFFECT_Ptr = { 
    read=FGetInternal_DIEFFECT_Ptr, nodefault }; 

    void __fastcall Internal_DIEFFECT_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Object Inspector 'Flag' Access Methods
    // ----------------------------------------------------------------------
    TDIEffect_Flags_Set __fastcall FGetFlags_(); 
    void __fastcall FSetFlags_( TDIEffect_Flags_Set pFlags_ ); 

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword* __fastcall FGetAxes(); 
    void __fastcall FSetAxes( dword* pAxes ); 
    dword __fastcall FGetAxesCount(); 
    void __fastcall FSetAxesCount( dword pAxesCount ); 
    long* __fastcall FGetDirection(); 
    void __fastcall FSetDirection( long* pDirection ); 
    dword __fastcall FGetDuration(); 
    void __fastcall FSetDuration( dword pDuration ); 
    TDIEnvelope* __fastcall FGetEnvelope(); 
    void __fastcall FSetEnvelope( TDIEnvelope* pEnvelope ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetFlags(); 
    void __fastcall FSetFlags( dword pFlags ); 
    dword __fastcall FGetGain(); 
    void __fastcall FSetGain( dword pGain ); 
    dword __fastcall FGetSamplePeriod(); 
    void __fastcall FSetSamplePeriod( dword pSamplePeriod ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    dword __fastcall FGetStartDelay(); 
    void __fastcall FSetStartDelay( dword pStartDelay ); 
    dword __fastcall FGetTriggerButton(); 
    void __fastcall FSetTriggerButton( dword pTriggerButton ); 
    dword __fastcall FGetTriggerRepeatInterval(); 
    void __fastcall FSetTriggerRepeatInterval( dword pTriggerRepeatInterval 
    ); 
    void* __fastcall FGetTypeSpecificParams(); 
    void __fastcall FSetTypeSpecificParams( void* pTypeSpecificParams ); 
    dword __fastcall FGetTypeSpecificParamsSize(); 
    void __fastcall FSetTypeSpecificParamsSize( dword pTypeSpecificParamsSize 
    ); 

    // ----------------------------------------------------------------------
    // Flag_Strings Access Methods
    // ----------------------------------------------------------------------
    TStringList* __fastcall FGetFlags_Strings();

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DIEFFECT* __fastcall FGetInternal_DIEFFECT_Ptr(); 
    void __fastcall FSetInternal_DIEFFECT( DIEFFECT* pDIEFFECT ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    TDIEnvelope* fEnvelope; 
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Flags_Strings Variables
    // ----------------------------------------------------------------------
    TStringList* fFlags_Strings;

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DIEFFECT fDIEFFECT; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
