#ifndef TDIFileEffectH
#define TDIFileEffectH

// ==========================================================================
// File:         TDIFileEffect.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons, website),
//               Brian Austin (some source code, documentation)
// Description:  This file defines the TDIFileEffect Component 
//
// "TDx_Input_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Input_Library_Defns.H"
#include "TDx_Input_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by TDIFileEffect properties.
#include "TDIEffect.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDIFileEffect
// Description: The TDIFileEffect component wraps multiple DirectX 
//              DIFILEEFFECT structures, which describe a force feedback 
//              effect stored or being written to a file. 
//              
//              This component is used by the 
//              TDx_InputDevice::WriteEffectToFile() and 
//              TDx_InputDevice::EnumEffectsInFile() methods when saving and 
//              retrieving effects. 
//              
//              To setup this component, first set the 
//              TDIFileEffect::ArraySize property, then set each property 
//              using array indexes, eg: 
//              
//                  // create internal storage for 5 DIFILEEFFECT structures 
//              
//                  DIFileEffect1->ArraySize = 5; 
//              
//                  // fill each array with appropriate data 
//              
//                  for (int i=0;i<5;i++) 
//                      { 
//                      DIFileEffect->FriendlyName[i] = "Chester Tester 
//              #"+IntToStr(i); 
//                      } 
//              
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDIFileEffect : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDIFileEffect : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDIFileEffect component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ArraySize
    // Description: The ArraySize property contains the number of internal 
    //              DIFILEEFFECT structures wrapped inside the TDIFileEffect 
    //              component. 
    //              By default the ArraySize is initialized to 1, meaning 
    //              that this TDIFileEffect component wraps a single 
    //              DIFILEEFFECT structure. To wrap multiple structures, set 
    //              the TDIFileEffect::ArraySize to the total number of 
    //              internal structures desired. 
    // ----------------------------------------------------------------------
    __property dword ArraySize = { read=FGetArraySize, write=FSetArraySize, 
    default=1 }; 

    // ----------------------------------------------------------------------
    // Property:    Effect
    // Description: The Effect property references a TDIEffect component 
    //              holding the effect information. 
    // ----------------------------------------------------------------------
    __property TDIEffect* Effect[dword pArrayIndex] = { read=FGetEffect, 
    write=FSetEffect }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDIFileEffect method or 
    //              fget/fset. eg. DI_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DI_OK }; 

    // ----------------------------------------------------------------------
    // Property:    FriendlyName
    // Description: The FriendlyName property defines a friendly name for the 
    //              effect such as 'Sawtooth up' or 'Constant force'. 
    // ----------------------------------------------------------------------
    __property AnsiString FriendlyName[dword pArrayIndex] = { 
    read=FGetFriendlyName, write=FSetFriendlyName }; 

    // ----------------------------------------------------------------------
    // Property:    GuidEffect
    // Description: The GuidEffect property identifies the effect type. 
    //              
    //              This may be one of the standard GUIDs listed below, or 
    //              one created by the effects designer. 
    //              
    //              GUID_ConstantForce 
    //              GUID_RampForce 
    //              GUID_Square 
    //              GUID_Sine 
    //              GUID_Triangle 
    //              GUID_SawtoothUp 
    //              GUID_SawtoothDown 
    //              GUID_Spring 
    //              GUID_Damper 
    //              GUID_Inertia 
    //              GUID_Friction 
    //              GUID_CustomForce 
    // ----------------------------------------------------------------------
    __property GUID GuidEffect[dword pArrayIndex] = { read=FGetGuidEffect, 
    write=FSetGuidEffect }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of a single 
    //              internal DIFILEEFFECT structure. 
    // ----------------------------------------------------------------------
    __property dword Size[dword pArrayIndex] = { read=FGetSize, 
    write=FSetSize }; 

    // ----------------------------------------------------------------------
    // Property:    SizeAll
    // Description: The SizeAll property contains the size in bytes of all 
    //              internal DIFILEEFFECT structures. This result is the same 
    //              as multiplying TDIFileEffect::ArraySize and 
    //              TDIFileEffect::Size. 
    // ----------------------------------------------------------------------
    __property dword SizeAll = { read=FGetSizeAll, nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDIFileEffect's internal DIFILEEFFECT structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // Params:      pArrayIndex - 
    //                The ArrayIndex parameter determines which internal 
    //                structure needs to be cleared. Correct values range from 
    //                0 to TDIFileEffect::ArraySize-1. 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear( dword pArrayIndex ); 

    // ----------------------------------------------------------------------
    // Method:      ClearAll()
    // Description: The ClearAll() method can be used to clear the contents 
    //              of all the internal DIFILEEFFECT structures of the 
    //              TDIFileEffect component. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall ClearAll(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDIFileEffect(TComponent* Owner); 
    virtual __fastcall ~TDIFileEffect(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DIFILEEFFECT* Internal_DIFILEEFFECT_Ptr = { 
    read=FGetInternal_DIFILEEFFECT_Ptr, nodefault }; 

    void __fastcall Internal_DIFILEEFFECT_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetArraySize(); 
    void __fastcall FSetArraySize( dword pArraySize ); 
    TDIEffect* __fastcall FGetEffect( dword pArrayIndex ); 
    void __fastcall FSetEffect( dword pArrayIndex, TDIEffect* pEffect ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    AnsiString __fastcall FGetFriendlyName( dword pArrayIndex ); 
    void __fastcall FSetFriendlyName( dword pArrayIndex, AnsiString 
    pFriendlyName ); 
    GUID __fastcall FGetGuidEffect( dword pArrayIndex ); 
    void __fastcall FSetGuidEffect( dword pArrayIndex, GUID pGuidEffect ); 
    dword __fastcall FGetSize( dword pArrayIndex ); 
    void __fastcall FSetSize( dword pArrayIndex, dword pSize ); 
    dword __fastcall FGetSizeAll(); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DIFILEEFFECT* __fastcall FGetInternal_DIFILEEFFECT_Ptr(); 
    void __fastcall FSetInternal_DIFILEEFFECT( dword pArrayIndex, 
    DIFILEEFFECT* pDIFILEEFFECT ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    dword fArraySize; 
    TDIEffect* fEffect; 
    HRESULT fErrorValue; 
    GUID fGuidEffect; 
    dword fSizeAll; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DIFILEEFFECT* fDIFILEEFFECT; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
