#ifndef TDIJoyState2H
#define TDIJoyState2H

// ==========================================================================
// File:         TDIJoyState2.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons, website),
//               Brian Austin (some source code, documentation)
// Description:  This file defines the TDIJoyState2 Component 
//
// "TDx_Input_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Input_Library_Defns.H"
#include "TDx_Input_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDIJoyState2
// Description: The TDIJoyState2 component wraps the DirectX DIJOYSTATE 
//              structure which holds information about the state of an 
//              extended joystick device. 
//              
//              This component is used with the 
//              TDx_InputDevice::GetDeviceState() method to obtain current 
//              device information, and with the 
//              TDx_InputDevice::SetDataFormat() method to set the devices 
//              data format. 
//              
//              When axes are in relative mode, the appropriate properties 
//              will describe the change since last reported. 
//              When axes are in absolute mode, the appropriate properties 
//              will describe the position relative to a fixed point. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDIJoyState2 : public TDIData
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDIJoyState2 : public TDIData
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ARX
    // Description: The ARX property describes the angular acceleration 
    //              around the joysticks X axis. 
    //              
    //              When angular acceleration is unsupported, this value will 
    //              be 0. 
    // ----------------------------------------------------------------------
    __property long ARX = { read=FGetARX, write=FSetARX, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    ARY
    // Description: The ARY property describes the angular acceleration 
    //              around the joysticks Y axis. 
    //              
    //              When angular acceleration is unsupported, this value will 
    //              be 0. 
    // ----------------------------------------------------------------------
    __property long ARY = { read=FGetARY, write=FSetARY, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    ARZ
    // Description: The ARZ property describes the angular acceleration 
    //              around the joysticks Z axis. 
    //              
    //              When angular acceleration is unsupported or there is no Z 
    //              axis, this value will be 0. 
    // ----------------------------------------------------------------------
    __property long ARZ = { read=FGetARZ, write=FSetARZ, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    ASlider
    // Description: The ASlider property defines an long array describing 
    //              acceration along the two additional axes described in 
    //              TDIJoystate2::Slider. 
    // ----------------------------------------------------------------------
    __property long ASlider[dword pIndex] = { read=FGetASlider, 
    write=FSetASlider }; 

    // ----------------------------------------------------------------------
    // Property:    AX
    // Description: The AX property describes acceleration along the 
    //              joysticks X axis. 
    //              
    //              When acceleration is unsupported, this value will be 0. 
    // ----------------------------------------------------------------------
    __property long AX = { read=FGetAX, write=FSetAX, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    AY
    // Description: The AY property describes acceleration along the 
    //              joysticks Y axis. 
    //              
    //              When acceleration is unsupported, this value will be 0. 
    // ----------------------------------------------------------------------
    __property long AY = { read=FGetAY, write=FSetAY, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    AZ
    // Description: The AZ property describes acceleration along the 
    //              joysticks Z axis. 
    //              
    //              When acceleration is unsupported or there is no Z axis, 
    //              this value will be 0. 
    // ----------------------------------------------------------------------
    __property long AZ = { read=FGetAZ, write=FSetAZ, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    FRX
    // Description: The FRX property describes the rotational force (torque) 
    //              on the joysticks X axis. 
    //              
    //              When torque is unsupported, this value will be 0. 
    // ----------------------------------------------------------------------
    __property long FRX = { read=FGetFRX, write=FSetFRX, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    FRY
    // Description: The FRY property describes the rotational force (torque) 
    //              on the joysticks Y axis. 
    //              
    //              When torque is unsupported, this value will be 0. 
    // ----------------------------------------------------------------------
    __property long FRY = { read=FGetFRY, write=FSetFRY, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    FRZ
    // Description: The FRZ property describes the rotational force (torque) 
    //              on the joysticks Z axis. 
    //              
    //              When torque is unsupported or there is no Z axis, this 
    //              value will be 0. 
    // ----------------------------------------------------------------------
    __property long FRZ = { read=FGetFRZ, write=FSetFRZ, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    FSlider
    // Description: The FSlider property defines an long array describing the 
    //              force on the two additional axes described in 
    //              TDIJoystate2::Slider. 
    // ----------------------------------------------------------------------
    __property long FSlider[dword pIndex] = { read=FGetFSlider, 
    write=FSetFSlider }; 

    // ----------------------------------------------------------------------
    // Property:    FX
    // Description: The FX property describes the force on the joysticks X 
    //              axis. 
    //              
    //              When force is unsupported, this value will be 0. 
    // ----------------------------------------------------------------------
    __property long FX = { read=FGetFX, write=FSetFX, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    FY
    // Description: The FY property describes the force on the joysticks Y 
    //              axis. 
    //              
    //              When force is unsupported, this value will be 0. 
    // ----------------------------------------------------------------------
    __property long FY = { read=FGetFY, write=FSetFY, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    FZ
    // Description: The FZ property describes the force on the joysticks Z 
    //              axis. 
    //              
    //              When force is unsupported or there is no Z axis, this 
    //              value will be 0. 
    // ----------------------------------------------------------------------
    __property long FZ = { read=FGetFZ, write=FSetFZ, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    POV
    // Description: The POV property defines a dword array indicating the 
    //              position, in 100ths of a degree, of up to 4 direction 
    //              controllers such as Point Of View hats. 
    //              
    //              Direction is measured clockwise from directly away from 
    //              the user and the centered position is reported as -1 or 
    //              65535. 
    //              
    //              Thus, for an indicator with five positions, it could have 
    //              a value of -1(or 65535) (centered), 0 (away from user), 
    //              9,000 (to the right), 18,000 (towards the user), or 
    //              27,000 (to the left). 
    // ----------------------------------------------------------------------
    __property dword POV[dword pIndex] = { read=FGetPOV, write=FSetPOV }; 

    // ----------------------------------------------------------------------
    // Property:    RX
    // Description: The RX property describes the rotation of the joysticks X 
    //              axis. 
    //              
    //              When rotation is unsupported, this value will be 0. 
    // ----------------------------------------------------------------------
    __property long RX = { read=FGetRX, write=FSetRX, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    RY
    // Description: The RY property describes the rotation of the joysticks Y 
    //              axis. 
    //              
    //              When rotation is unsupported, this value will be 0. 
    // ----------------------------------------------------------------------
    __property long RY = { read=FGetRY, write=FSetRY, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    RZ
    // Description: The RZ property describes the rotation of the joysticks Z 
    //              axis. 
    //              
    //              When rotation is unsupported or there is no Z axis, this 
    //              value will be 0. 
    // ----------------------------------------------------------------------
    __property long RZ = { read=FGetRZ, write=FSetRZ, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Slider
    // Description: The Slider property defines an long array describing two 
    //              additional axis values whose semantics depend on the 
    //              specific joystick being used. 
    //              
    //              TDx_InputDevice::GetObjectInfo() can be used to obtain 
    //              semantic information about these axes, which were 
    //              formally called the U and V axes. 
    // ----------------------------------------------------------------------
    __property long Slider[dword pIndex] = { read=FGetSlider, 
    write=FSetSlider }; 

    // ----------------------------------------------------------------------
    // Property:    VRX
    // Description: The VRX property describes the angular velocity around 
    //              the joysticks X axis. 
    //              
    //              When angular velocity is unsupported, this value will be 
    //              0. 
    // ----------------------------------------------------------------------
    __property long VRX = { read=FGetVRX, write=FSetVRX, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    VRY
    // Description: The VRY property describes the angular velocity around 
    //              the joysticks Y axis. 
    //              
    //              When angular velocity is unsupported, this value will be 
    //              0. 
    // ----------------------------------------------------------------------
    __property long VRY = { read=FGetVRY, write=FSetVRY, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    VRZ
    // Description: The VRZ property describes the angular velocity around 
    //              the joysticks Z axis. 
    //              
    //              When angular velocity is unsupported or there is no Z 
    //              axis, this value will be 0. 
    // ----------------------------------------------------------------------
    __property long VRZ = { read=FGetVRZ, write=FSetVRZ, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    VSlider
    // Description: The VSlider property defines an long array describing 
    //              velocity along the two additional axes described in 
    //              TDIJoystate2::Slider. 
    // ----------------------------------------------------------------------
    __property long VSlider[dword pIndex] = { read=FGetVSlider, 
    write=FSetVSlider }; 

    // ----------------------------------------------------------------------
    // Property:    VX
    // Description: The VX property describes the velocity along the 
    //              joysticks X axis. 
    //              
    //              When velocity is unsupported, this value will be 0. 
    // ----------------------------------------------------------------------
    __property long VX = { read=FGetVX, write=FSetVX, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    VY
    // Description: The VY property describes the velocity along the 
    //              joysticks Y axis. 
    //              
    //              When velocity is unsupported, this value will be 0. 
    //              
    // ----------------------------------------------------------------------
    __property long VY = { read=FGetVY, write=FSetVY, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    VZ
    // Description: The VZ property describes the velocity along the 
    //              joysticks Z axis. 
    //              
    //              When velocity is unsupported or there is no Z axis, this 
    //              value will be 0. 
    // ----------------------------------------------------------------------
    __property long VZ = { read=FGetVZ, write=FSetVZ, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    X
    // Description: The X property describes position information for the 
    //              joysticks X axis. 
    //              
    //              This axis usually represents the left right movement of a 
    //              control stick. 
    // ----------------------------------------------------------------------
    __property long X = { read=FGetX, write=FSetX, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Y
    // Description: The Y property describes position information for the 
    //              joysticks Y axis. 
    //              
    //              This axis usually represents the backward forward 
    //              movement of a control stick. 
    // ----------------------------------------------------------------------
    __property long Y = { read=FGetY, write=FSetY, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Z
    // Description: The Z property describes position information for the 
    //              joysticks Z axis. 
    //              
    //              This axis often represents a throttle control of a 
    //              joystick. 
    //              If there is no Z axis, this value will be 0. 
    // ----------------------------------------------------------------------
    __property long Z = { read=FGetZ, write=FSetZ, default=0 }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDIJoyState2 component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Buttons
    // Description: The Buttons property defines a byte array of up to 128 
    //              button states. 
    //              
    //              When a given button is down, the high-order bit of the 
    //              corresponding byte will be set. 
    //              If the button is up, or does not exist, the bit will be 
    //              clear. 
    // ----------------------------------------------------------------------
    __property byte Buttons[dword pIndex] = { read=FGetButtons, 
    write=FSetButtons }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDIJoyState2 method or 
    //              fget/fset. eg. DI_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DI_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of the 
    //              internal DIJOYSTATE2 structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDIJoyState2's internal DIJOYSTATE2 structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDIJoyState2(TComponent* Owner); 
    virtual __fastcall ~TDIJoyState2(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DIJOYSTATE2* Internal_DIJOYSTATE2_Ptr = { 
    read=FGetInternal_DIJOYSTATE2_Ptr, nodefault }; 

    void __fastcall Internal_DIJOYSTATE2_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    long __fastcall FGetARX(); 
    void __fastcall FSetARX( long pARX ); 
    long __fastcall FGetARY(); 
    void __fastcall FSetARY( long pARY ); 
    long __fastcall FGetARZ(); 
    void __fastcall FSetARZ( long pARZ ); 
    long __fastcall FGetASlider( dword pIndex ); 
    void __fastcall FSetASlider( dword pIndex, long pASlider ); 
    long __fastcall FGetAX(); 
    void __fastcall FSetAX( long pAX ); 
    long __fastcall FGetAY(); 
    void __fastcall FSetAY( long pAY ); 
    long __fastcall FGetAZ(); 
    void __fastcall FSetAZ( long pAZ ); 
    byte __fastcall FGetButtons( dword pIndex ); 
    void __fastcall FSetButtons( dword pIndex, byte pButtons ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    long __fastcall FGetFRX(); 
    void __fastcall FSetFRX( long pFRX ); 
    long __fastcall FGetFRY(); 
    void __fastcall FSetFRY( long pFRY ); 
    long __fastcall FGetFRZ(); 
    void __fastcall FSetFRZ( long pFRZ ); 
    long __fastcall FGetFSlider( dword pIndex ); 
    void __fastcall FSetFSlider( dword pIndex, long pFSlider ); 
    long __fastcall FGetFX(); 
    void __fastcall FSetFX( long pFX ); 
    long __fastcall FGetFY(); 
    void __fastcall FSetFY( long pFY ); 
    long __fastcall FGetFZ(); 
    void __fastcall FSetFZ( long pFZ ); 
    dword __fastcall FGetPOV( dword pIndex ); 
    void __fastcall FSetPOV( dword pIndex, dword pPOV ); 
    long __fastcall FGetRX(); 
    void __fastcall FSetRX( long pRX ); 
    long __fastcall FGetRY(); 
    void __fastcall FSetRY( long pRY ); 
    long __fastcall FGetRZ(); 
    void __fastcall FSetRZ( long pRZ ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    long __fastcall FGetSlider( dword pIndex ); 
    void __fastcall FSetSlider( dword pIndex, long pSlider ); 
    long __fastcall FGetVRX(); 
    void __fastcall FSetVRX( long pVRX ); 
    long __fastcall FGetVRY(); 
    void __fastcall FSetVRY( long pVRY ); 
    long __fastcall FGetVRZ(); 
    void __fastcall FSetVRZ( long pVRZ ); 
    long __fastcall FGetVSlider( dword pIndex ); 
    void __fastcall FSetVSlider( dword pIndex, long pVSlider ); 
    long __fastcall FGetVX(); 
    void __fastcall FSetVX( long pVX ); 
    long __fastcall FGetVY(); 
    void __fastcall FSetVY( long pVY ); 
    long __fastcall FGetVZ(); 
    void __fastcall FSetVZ( long pVZ ); 
    long __fastcall FGetX(); 
    void __fastcall FSetX( long pX ); 
    long __fastcall FGetY(); 
    void __fastcall FSetY( long pY ); 
    long __fastcall FGetZ(); 
    void __fastcall FSetZ( long pZ ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DIJOYSTATE2* __fastcall FGetInternal_DIJOYSTATE2_Ptr(); 
    void __fastcall FSetInternal_DIJOYSTATE2( DIJOYSTATE2* pDIJOYSTATE2 ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    dword fSize; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DIJOYSTATE2 fDIJOYSTATE2; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
