#ifndef TDIMouseStateH
#define TDIMouseStateH

// ==========================================================================
// File:         TDIMouseState.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons, website),
//               Brian Austin (some source code, documentation)
// Description:  This file defines the TDIMouseState Component 
//
// "TDx_Input_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Input_Library_Defns.H"
#include "TDx_Input_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDIMouseState
// Description: The TDIMouseState component wraps the DirectX DIMOUSESTATE 
//              structure which holds information about the state of a basic 
//              mouse device. 
//              
//              This component is used with the 
//              TDx_InputDevice::GetDeviceState() method to obtain current 
//              device information, and with the 
//              TDx_InputDevice::SetDataFormat() method to set the devices 
//              data format. 
//              
//              When axes are in relative mode, the appropriate properties 
//              will describe the change since last reported. 
//              When axes are in absolute mode, the appropriate properties 
//              will describe the position relative to a fixed point. 
//              
//              Absolute axis information for a mouse is simple accumulated 
//              relative motion. This means the absolute axis position is 
//              only useful when comparing it with other absolute axis 
//              positions. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDIMouseState : public TDIData
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDIMouseState : public TDIData
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    X
    // Description: The X property describes movement of the mouse's X axis, 
    //              which usually represents the left right movement of the 
    //              mouse. 
    // ----------------------------------------------------------------------
    __property long X = { read=FGetX, write=FSetX, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Y
    // Description: The Y property describes movement of the mouse's Y axis, 
    //              which usually represents the forward backward movement of 
    //              the mouse. 
    // ----------------------------------------------------------------------
    __property long Y = { read=FGetY, write=FSetY, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Z
    // Description: The Z property describes movement of the mouse's Z axis, 
    //              which is usually a wheel between the buttons. 
    //              
    //              When there is no Z axis this value will be 0. 
    // ----------------------------------------------------------------------
    __property long Z = { read=FGetZ, write=FSetZ, default=0 }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDIMouseState component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Buttons
    // Description: The Buttons property defines a byte array of up to 4 
    //              button states. 
    //              
    //              When a given button is down, the high-order bit of the 
    //              corresponding byte will be set. 
    //              If the button is up, or does not exist, the bit will be 
    //              clear. 
    // ----------------------------------------------------------------------
    __property byte Buttons[dword pIndex] = { read=FGetButtons, 
    write=FSetButtons }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDIMouseState method or 
    //              fget/fset. eg. DI_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DI_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of the 
    //              internal DIMOUSESTATE structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDIMouseState's internal DIMOUSESTATE structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDIMouseState(TComponent* Owner); 
    virtual __fastcall ~TDIMouseState(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DIMOUSESTATE* Internal_DIMOUSESTATE_Ptr = { 
    read=FGetInternal_DIMOUSESTATE_Ptr, nodefault }; 

    void __fastcall Internal_DIMOUSESTATE_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    byte __fastcall FGetButtons( dword pIndex ); 
    void __fastcall FSetButtons( dword pIndex, byte pButtons ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    long __fastcall FGetX(); 
    void __fastcall FSetX( long pX ); 
    long __fastcall FGetY(); 
    void __fastcall FSetY( long pY ); 
    long __fastcall FGetZ(); 
    void __fastcall FSetZ( long pZ ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DIMOUSESTATE* __fastcall FGetInternal_DIMOUSESTATE_Ptr(); 
    void __fastcall FSetInternal_DIMOUSESTATE( DIMOUSESTATE* pDIMOUSESTATE ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    dword fSize; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DIMOUSESTATE fDIMOUSESTATE; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
