#ifndef TDIPeriodicH
#define TDIPeriodicH

// ==========================================================================
// File:         TDIPeriodic.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons, website),
//               Brian Austin (some source code, documentation)
// Description:  This file defines the TDIPeriodic Component 
//
// "TDx_Input_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Input_Library_Defns.H"
#include "TDx_Input_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDIPeriodic
// Description: The TDIPeriodic component wraps the DirectX DIPERIODIC 
//              structure which contains type specific information for force 
//              feedback effects that have the DIEFT_PERIODIC option set. 
//              
//              This component is referenced by TDIEffect::TypeSpecificParams 
//              when it is being used to define a periodic effect. 
//              
//              Periodic effects are waveforms that are usually used to 
//              achieve a shaking in the control device. You can apply an 
//              envelope to a periodic effect. 
//              
//              To create a standard periodic effect pass one of the GUIDs 
//              below in the TDx_InputDevice::CreateEffect() Guid parameter. 
//              
//              The standard waveform GUIDs are: 
//              
//                  GUID_Square       - Feedback oscillates directly from 
//              maximum negative to maximum positive force. 
//                  GUID_Sine         -  Feedback oscillates using a sine 
//              waveform pattern of smooth curves. 
//                  GUID_Triangle     - Feedback oscillates from positive to 
//              negative at a constant rate of force change. 
//                  GUID_SawtoothUp   - Feedback increases at a fixed rate 
//              then drops directly to the maximum negative magnitude and 
//              begins increasing again. 
//                  GUID_SawtoothDown - Feedback decreases at a fixed rate 
//              then rises directly to the maximum positive magnitude and 
//              begins decreasing again. 
//              
//              Magnitude is defined here, while the duration is defined by 
//              TDIEffect::Duration. 
//              
//              A periodic force using an envelope will start its cycle at 
//              the attack level, relative to its phase, spend a specified 
//              duration moving towards the sustain level, stay at the 
//              sustain level for a specified period, then spend another 
//              period of time moving towards the fade level. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDIPeriodic : public TDIForce
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDIPeriodic : public TDIForce
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Magnitude
    // Description: The Magnitude property defines the maximum amplitude of 
    //              the waveform from the TDIPeriodic::Offset value. 
    //              
    //              This value is in a range from 0, representing no force 
    //              output at all, to 10,000, representing the maximum force 
    //              output possible and represents the sustain magnitude if 
    //              an envelope is being used. 
    // ----------------------------------------------------------------------
    __property dword Magnitude = { read=FGetMagnitude, write=FSetMagnitude, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Offset
    // Description: The Offset property defines the offset for the effect, 
    //              which represents a baseline to which the value of 
    //              TDIPeriodic::Magnitude is applied in both directions to 
    //              establish the actual directional force outputs for the 
    //              effect. 
    //              
    //              This value can range from -10,000 to 10,000, the extreme 
    //              values representing the maximum force output the device 
    //              is capable of in the positive and negative directions. 
    //              
    //              For example, an offset of 2000 and a magnitude of 5000 
    //              for a periodic effect along the X axis would result in a 
    //              force that oscillates between 7000 and -3000. 
    // ----------------------------------------------------------------------
    __property long Offset = { read=FGetOffset, write=FSetOffset, default=0 
    }; 

    // ----------------------------------------------------------------------
    // Property:    Period
    // Description: The Period property defines the how long, in 
    //              microseconds, the waveform will take to do one complete 
    //              cycle. 
    // ----------------------------------------------------------------------
    __property dword Period = { read=FGetPeriod, write=FSetPeriod, default=0 
    }; 

    // ----------------------------------------------------------------------
    // Property:    Phase
    // Description: The Phase property defines the position, in 100ths of a 
    //              degree, in the waveform at which playback begins. 
    //              
    //              This property has a range of 0 to 35999 but all values 
    //              may not be supported, in which case it will be rounded to 
    //              the nearest value that is supported. 
    // ----------------------------------------------------------------------
    __property dword Phase = { read=FGetPhase, write=FSetPhase, default=0 }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDIPeriodic component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDIPeriodic method or 
    //              fget/fset. eg. DI_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DI_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of the 
    //              internal DIPERIODIC structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDIPeriodic's internal DIPERIODIC structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDIPeriodic(TComponent* Owner); 
    virtual __fastcall ~TDIPeriodic(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DIPERIODIC* Internal_DIPERIODIC_Ptr = { 
    read=FGetInternal_DIPERIODIC_Ptr, nodefault }; 

    void __fastcall Internal_DIPERIODIC_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetMagnitude(); 
    void __fastcall FSetMagnitude( dword pMagnitude ); 
    long __fastcall FGetOffset(); 
    void __fastcall FSetOffset( long pOffset ); 
    dword __fastcall FGetPeriod(); 
    void __fastcall FSetPeriod( dword pPeriod ); 
    dword __fastcall FGetPhase(); 
    void __fastcall FSetPhase( dword pPhase ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DIPERIODIC* __fastcall FGetInternal_DIPERIODIC_Ptr(); 
    void __fastcall FSetInternal_DIPERIODIC( DIPERIODIC* pDIPERIODIC ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    dword fSize; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DIPERIODIC fDIPERIODIC; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
