#ifndef TDIPropDWORDH
#define TDIPropDWORDH

// ==========================================================================
// File:         TDIPropDWORD.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons, website),
//               Brian Austin (some source code, documentation)
// Description:  This file defines the TDIPropDWORD Component 
//
// "TDx_Input_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Input_Library_Defns.H"
#include "TDx_Input_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by TDIPropDWORD properties.
#include "TDIPropHeader.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDIPropDWORD
// Description: The TDIPropDWORD component wraps the DirectX DIPROPDWORD 
//              structure which is a generic structure used to access dword 
//              properties. 
//              
//              This component is primarily utilised by the 
//              TDx_InputDevice::GetProperty() and 
//              TDx_InputDevice::SetProperty() methods to access properties 
//              that require a single value, such as a buffer size. 
//              
//              This component also references a TDIPropHeader component. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDIPropDWORD : public TDIProperty
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDIPropDWORD : public TDIProperty
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Data
    // Description: The Data property defines value being set or retrieved. 
    // ----------------------------------------------------------------------
    __property dword Data = { read=FGetData, write=FSetData, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    PropHeader
    // Description: The PropHeader property can be used to link the 
    //              TDIPropDWORD to a TDIPropHeader component using the 
    //              Object Inspector or equivalent runtime code, and is used 
    //              to access data inside the DIPROPHEADER structure that 
    //              exists inside a DIPROPDWORD structure in a 
    //              structure-in-structure relationship. 
    //              
    //              Normally in DirectX, you must setup the entire 
    //              DIPROPDWORD structure including it's DIPROPHEADER 
    //              structure according to specific rules. Upon linking a 
    //              TDIPropHeader component to a TDIPropDWORD, much of the 
    //              work has already been performed, but you still need to 
    //              setup both the TDIPropHeader::Obj and the 
    //              TDIPropHeader::How properties. 
    //              
    //              For Example :- 
    //              
    //              void __fastcall Form1::Button1Click(TObject* Sender) 
    //              { 
    //              DIPropHeader1->How = DIPH_DEVICE; 
    //              DIPropHeader1->Obj = 0; 
    //              
    //              DIPropDWORD1->Data = 20; 
    //              DIPropDWORD1->PropHeader = DIPropHeader1; 
    //              
    //              Dx_InputDevice1->SetProperty( DIPROP_BUFFERSIZE, 
    //              DIPropDWORD1 ); 
    //              } 
    // ----------------------------------------------------------------------
    __property TDIPropHeader* PropHeader = { read=FGetPropHeader, 
    write=FSetPropHeader, default=NULL }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDIPropDWORD component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDIPropDWORD method or 
    //              fget/fset. eg. DI_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DI_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of the 
    //              internal DIPROPDWORD structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDIPropDWORD's internal DIPROPDWORD structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDIPropDWORD(TComponent* Owner); 
    virtual __fastcall ~TDIPropDWORD(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DIPROPDWORD* Internal_DIPROPDWORD_Ptr = { 
    read=FGetInternal_DIPROPDWORD_Ptr, nodefault }; 

    void __fastcall Internal_DIPROPDWORD_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetData(); 
    void __fastcall FSetData( dword pData ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    TDIPropHeader* __fastcall FGetPropHeader(); 
    void __fastcall FSetPropHeader( TDIPropHeader* pPropHeader ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DIPROPDWORD* __fastcall FGetInternal_DIPROPDWORD_Ptr(); 
    void __fastcall FSetInternal_DIPROPDWORD( DIPROPDWORD* pDIPROPDWORD ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    TDIPropHeader* fPropHeader; 
    dword fSize; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DIPROPDWORD fDIPROPDWORD; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
