#ifndef TDIPropStringH
#define TDIPropStringH

// ==========================================================================
// File:         TDIPropString.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons, website),
//               Brian Austin (some source code, documentation)
// Description:  This file defines the TDIPropString Component 
//
// "TDx_Input_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Input_Library_Defns.H"
#include "TDx_Input_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by TDIPropString properties.
#include "TDIPropHeader.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDIPropString
// Description: The TDIPropString component wraps the DirectX TDIPROPSTRING 
//              structure which is used to access string properties. 
//              
//              This component is primarily utilised by the 
//              TDx_InputDevice::GetProperty() and 
//              TDx_InputDevice::SetProperty() methods to perform operations 
//              on a HID that are not otherwise supported. 
//              
//              This component also references a TDIPropHeader component. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDIPropString : public TDIProperty
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDIPropString : public TDIProperty
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    PropHeader
    // Description: The PropHeader property can be used to link the 
    //              TDIPropString to a TDIPropHeader component using the 
    //              Object Inspector or equivalent runtime code, and is used 
    //              to access data inside the DIPROPHEADER structure that 
    //              exists inside a DIPROPSTRING structure in a 
    //              structure-in-structure relationship. 
    //              
    //              Normally in DirectX, you must setup the entire 
    //              DIPROPSTRING structure including it's DIPROPHEADER 
    //              structure according to specific rules. Upon linking a 
    //              TDIPropHeader component to a TDIPropString, much of the 
    //              work has already been performed, but you still need to 
    //              setup both the TDIPropHeader::Obj and the 
    //              TDIPropHeader::How properties. 
    //              
    //              For Example :- 
    //              
    //              void __fastcall Form1::Button1Click(TObject* Sender) 
    //              { 
    //              DIPropHeader1->How = DIPH_DEVICE; 
    //              DIPropHeader1->Obj = 0; 
    //              
    //              DIPropString1->String = "My Device"; 
    //              DIPropString1->PropHeader = DIPropHeader1; 
    //              
    //              Dx_InputDevice1->SetProperty( DIPROP_PRODUCTNAME, 
    //              DIPropString1 ); 
    //              } 
    // ----------------------------------------------------------------------
    __property TDIPropHeader* PropHeader = { read=FGetPropHeader, 
    write=FSetPropHeader, default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    String
    // Description: The String property defines the instance or product name 
    //              as a string. 
    // ----------------------------------------------------------------------
    __property AnsiString String = { read=FGetString, write=FSetString, 
    nodefault }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDIPropString component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDIPropString method or 
    //              fget/fset. eg. DI_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DI_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of the 
    //              internal DIPROPSTRING structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDIPropString's internal DIPROPSTRING structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDIPropString(TComponent* Owner); 
    virtual __fastcall ~TDIPropString(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DIPROPSTRING* Internal_DIPROPSTRING_Ptr = { 
    read=FGetInternal_DIPROPSTRING_Ptr, nodefault }; 

    void __fastcall Internal_DIPROPSTRING_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    TDIPropHeader* __fastcall FGetPropHeader(); 
    void __fastcall FSetPropHeader( TDIPropHeader* pPropHeader ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    AnsiString __fastcall FGetString(); 
    void __fastcall FSetString( AnsiString pString ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DIPROPSTRING* __fastcall FGetInternal_DIPROPSTRING_Ptr(); 
    void __fastcall FSetInternal_DIPROPSTRING( DIPROPSTRING* pDIPROPSTRING ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    TDIPropHeader* fPropHeader; 
    dword fSize; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DIPROPSTRING fDIPROPSTRING; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
