#ifndef TDIRampForceH
#define TDIRampForceH

// ==========================================================================
// File:         TDIRampForce.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons, website),
//               Brian Austin (some source code, documentation)
// Description:  This file defines the TDIRampForce Component 
//
// "TDx_Input_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Input_Library_Defns.H"
#include "TDx_Input_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDIRampForce
// Description: The TDIRampForce component wraps the DirectX DICONSTANTFORCE 
//              structure, which contains type specific information for force 
//              feedback effects that have the DIEFT_RAMPFORCE option set. 
//              
//              It is used to define a 'Ramp Force', which is a force that 
//              moves from one defined magnitude to another over a specified 
//              time period. 
//              
//              You can apply an envelope to a ramp force but not an offset. 
//              
//              Only the magnitudes are defined by this component, the 
//              duration is set in TDIEffect::Duration, which cannot be set 
//              to INFINITE for a ramp force. 
//              
//              Use this component by setting TDIEffect::TypeSpecificParams 
//              to refer to a single TDIRampForce component. 
//              
//              To create a ramp force, use the 
//              TDx_InputDevice::CreateEffect() method and pass the following 
//              as the GUID parameter: 
//              
//                  GUID_RampForce 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDIRampForce : public TDIForce
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDIRampForce : public TDIForce
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    End
    // Description: The End property defines the ending magnitude for the 
    //              effect. 
    //              
    //              This value can range from -10,000 to 10,000, the 
    //              extreemes of which represent the maximum force output the 
    //              device is capable of in either direction. 
    // ----------------------------------------------------------------------
    __property long End = { read=FGetEnd, write=FSetEnd, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Start
    // Description: The Start property defines the starting magnitude for the 
    //              effect. 
    //              
    //              This value can range from -10,000 to 10,000, the 
    //              extreemes of which represent the maximum force output the 
    //              device is capable of in either direction. 
    // ----------------------------------------------------------------------
    __property long Start = { read=FGetStart, write=FSetStart, default=0 }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDIRampForce component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDIRampForce method or 
    //              fget/fset. eg. DI_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DI_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of the 
    //              internal DIRAMPFORCE structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDIRampForce's internal DIRAMPFORCE structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDIRampForce(TComponent* Owner); 
    virtual __fastcall ~TDIRampForce(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DIRAMPFORCE* Internal_DIRAMPFORCE_Ptr = { 
    read=FGetInternal_DIRAMPFORCE_Ptr, nodefault }; 

    void __fastcall Internal_DIRAMPFORCE_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    long __fastcall FGetEnd(); 
    void __fastcall FSetEnd( long pEnd ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    long __fastcall FGetStart(); 
    void __fastcall FSetStart( long pStart ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DIRAMPFORCE* __fastcall FGetInternal_DIRAMPFORCE_Ptr(); 
    void __fastcall FSetInternal_DIRAMPFORCE( DIRAMPFORCE* pDIRAMPFORCE ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    dword fSize; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DIRAMPFORCE fDIRAMPFORCE; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
